# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio OAuth2Server Token dump functions."""

from __future__ import absolute_import, print_function

from .utils import dt2iso_or_empty


def get(*args, **kwargs):
    """Get users."""
    from invenio.modules.oauth2server.models import Token
    q = Token.query
    return q.count(), q.all()


def dump(obj, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump the oauth2server tokens."""
    return dict(id=obj.id,
                client_id=obj.client_id,
                user_id=obj.user_id,
                token_type=obj.token_type,
                access_token=obj.access_token,
                refresh_token=obj.refresh_token,
                expires=dt2iso_or_empty(obj.expires),
                _scopes=obj._scopes,
                is_personal=obj.is_personal,
                is_internal=obj.is_internal)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""CLI interface for Invenio 3 to load legacy dump."""

from __future__ import absolute_import, print_function

import json
import sys

import click
from celery import chain
from flask_cli import with_appcontext

from .proxies import current_migrator
from .tasks.records import import_record


@click.group()
def dumps():
    """Migration commands."""


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@click.option('--source-type', '-t',  type=click.Choice(['json', 'marcxml']),
              default='marcxml', help='Whether to use JSON or MARCXML.')
@with_appcontext
def loadrecords(sources, source_type):
    """Load records migration dump."""
    for idx, source in enumerate(sources, 1):
        click.echo('Loading dump {0} of {1} ({2})'.format(idx, len(sources),
                                                          source.name))
        data = json.load(source)
        with click.progressbar(data) as records:
            for item in records:
                if current_migrator.records_post_task:
                    chain(
                        import_record.s(item, source_type=source_type),
                        current_migrator.records_post_task.s()
                    )()
                else:
                    import_record.delay(item, source_type=source_type)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@click.option('--recid', type=int)
@click.option('--files', 'entity', flag_value='files')
@click.option('--json', 'entity', flag_value='json')
@click.option('--marcxml', 'entity', flag_value='marcxml')
@with_appcontext
def inspectrecords(sources, recid, entity=None):
    """Inspect records in a migration dump."""
    for idx, source in enumerate(sources, 1):
        click.echo('Loading dump {0} of {1} ({2})'.format(idx, len(sources),
                                                          source.name))
        data = json.load(source)

        # Just print record identifiers if none are selected.
        if not recid:
            click.secho('Record identifiers', fg='green')
            total = 0
            for r in (d['recid'] for d in data):
                click.echo(r)
                total += 1
            click.echo('{0} records found in dump.'.format(total))
            return

        data = list(filter(lambda d: d['recid'] == recid, data))

        if not data:
            click.secho("Record not found.", fg='yellow')
            return

        for record in data:
            if entity is None:
                click.echo(json.dumps(record, indent=2))
            if entity == 'files':
                click.secho('Files', fg='green')
                click.echo(
                    json.dumps(record['files'], indent=2))

            if entity == 'json':
                click.secho('Records (JSON)', fg='green')
                for revision in record['record']:
                    click.secho('Revision {0}'.format(
                        revision['modification_datetime']), fg='yellow')
                    click.echo(json.dumps(revision['json'], indent=2))

            if entity == 'marcxml':
                click.secho('Records (MARCXML)', fg='green')
                for revision in record['record']:
                    click.secho(
                        'Revision {0}'.format(revision['marcxml']),
                        fg='yellow')
                    click.echo(revision)


def loadcommon(sources, load_task, *args, **kwargs):
    """Common helper function for load simple objects.

    Note: Extra `args` and `kwargs` are passed to the `load_task` function.

    :param sources: JSON source files with dumps
    :type sources: list of str (filepaths)
    :param load_task: Shared task which loads the dump.
    :type load_task: function
    """
    click.echo('Loading dumps started.')
    for idx, source in enumerate(sources, 1):
        click.echo('Loading dump {0} of {1} ({2})'.format(idx, len(sources),
                                                          source.name))
        data = json.load(source)
        with click.progressbar(data) as data_bar:
            for d in data_bar:
                load_task.delay(d, *args, **kwargs)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@click.argument('logos_dir', type=click.Path(exists=True), default=None)
@with_appcontext
def loadcommunities(sources, logos_dir):
    """Load communities."""
    from invenio_migrator.tasks.communities import load_community
    loadcommon(sources, load_community, logos_dir)


@dumps.command()
@click.argument('sources', type=click.File('r'), default=sys.stdin)
@with_appcontext
def loadfeatured(sources):
    """Load community featurings."""
    from invenio_migrator.tasks.communities import load_featured
    loadcommon(sources, load_featured)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loadusers(sources):
    """Load users."""
    from .tasks.users import load_user
    loadcommon(sources, load_user)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loaddeposit(sources):
    """Load deposit."""
    from .tasks.deposit import load_deposit
    loadcommon(sources, load_deposit)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loadremoteaccounts(sources):
    """Load remote accounts."""
    from .tasks.oauthclient import load_remoteaccount
    loadcommon(sources, load_remoteaccount)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loadremotetokens(sources):
    """Load remote tokens."""
    from .tasks.oauthclient import load_remotetoken
    loadcommon(sources, load_remotetoken)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loaduserexts(sources):
    """Load user identities (legacy UserEXT)."""
    from .tasks.oauthclient import load_userext
    loadcommon(sources, load_userext)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loadtokens(sources):
    """Load server tokens."""
    from .tasks.oauth2server import load_token
    loadcommon(sources, load_token)


@dumps.command()
@click.argument('sources', type=click.File('r'), nargs=-1)
@with_appcontext
def loadclients(sources):
    """Load server clients."""
    from .tasks.oauth2server import load_client
    loadcommon(sources, load_client)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery task for records migration."""

from __future__ import absolute_import, print_function

from celery import shared_task
from invenio_oauthclient.models import RemoteAccount, RemoteToken, UserIdentity

from .utils import load_common


@shared_task()
def load_remoteaccount(data):
    """Load the remote accounts from data dump.

    :param data: Dictionary containing remote accounts data.
    :type data: dict
    """
    load_common(RemoteAccount, data)


@shared_task()
def load_remotetoken(data):
    """Load the remote tokens from data dump.

    :param data: Dictionary containing remote tokens data.
    :type data: dict
    """
    load_common(RemoteToken, data)


@shared_task()
def load_userext(data):
    """Load the user identities from UserEXT dump.

    :param data: Dictionary containing user identities.
    :type data: dict
    """
    load_common(UserIdentity, data)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

from invenio_migrator.tasks.oauth2server import load_client, load_token
from invenio_migrator.tasks.oauthclient import load_remoteaccount, \
    load_remotetoken


def test_oauthclient_load(dummy_location, oauthclient_dump, deposit_user, app):
    """Test the deposit loading function."""
    remoteaccounts, remotetokens = oauthclient_dump
    for ra in remoteaccounts:
        load_remoteaccount(ra)
    for rt in remotetokens:
        load_remotetoken(rt)


def test_oauth2server_load(dummy_location, oauth2server_dump, deposit_user,
                           app):
    """Test the deposit loading function."""
    clients, tokens = oauth2server_dump
    for client in clients:
        load_client(client)
    for token in tokens:
        load_token(token)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Test legacy record dump."""

from click.testing import CliRunner

from invenio_migrator.legacy.cli import dump as dump_cli
from invenio_migrator.legacy.records import (
    dump as dump_record,
    get as get_record
)


def test_dump_record():
    """Test single record dump."""
    record_dump = dump_record(
        10, '1970-01-01 00:00:00', latest_only=False,
        with_json=False, with_collections=True)

    assert record_dump

    assert record_dump['recid'] == 10
    assert record_dump['collections'] == [
        'ALEPH Papers',
        'Articles & Preprints',
        'Experimental Physics (EP)',
        'CERN Divisions',
        'Atlantis Institute of Fictive Science',
        'CERN Experiments',
        'Preprints',
        'ALEPH'
    ]
    assert len(record_dump['files']) == 2
    assert len(record_dump['record']) == 2
    assert 'marcxml' in record_dump['record'][0]
    assert 'json' in record_dump['record'][0]
    assert record_dump['record'][0]['json'] is None
    assert 'modification_datetime' in record_dump['record'][0]

    record_dump_latest = dump_record(
        10, '1970-01-01 00:00:00', with_json=False, latest_only=True)

    assert record_dump_latest['record'][0] == record_dump['record'][1]

    record_dump = dump_record(
        10, '1970-01-01 00:00:00', latest_only=False,
        with_json=False, with_collections=False)

    assert record_dump['collections'] is None

    record_dump = dump_record(
        10, '1970-01-01 00:00:00', latest_only=False,
        with_json=True, with_collections=True)
    assert record_dump['record'][0]['json']['recid'] == 10
    assert len(record_dump['record'][0]['json']['authors']) == 315


def test_get_record():
    """Test get record."""
    records = get_record('Ellis', '1970-01-01 00:00:00')
    assert records[0] == 13
    assert records[1] == set(
        [8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 118, 47])


def test_cli():
    """Test CLI."""
    runner = CliRunner()

    with runner.isolated_filesystem():
        result = runner.invoke(
            dump_cli, ['records', '-q', 'Ellis', '--with-json'])
        assert result.exit_code == 0



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery records tests."""

from __future__ import absolute_import, print_function

from invenio_files_rest.models import ObjectVersion
from invenio_records.models import RecordMetadata

from invenio_migrator.tasks.records import import_record


def test_import_record(app, db, dummy_location, record_dump, records_json,
                       resolver):
    """Test import record celery task."""
    assert RecordMetadata.query.count() == 0
    import_record(records_json[0], source_type='json')
    assert RecordMetadata.query.count() == 1
    pid, record = resolver.resolve('11782')
    assert record['_collections'] == []
    assert len(record['_files']) == 1
    assert ObjectVersion.get(
        record['_files'][0]['bucket'], record['_files'][0]['key'])

    import_record(records_json[1], source_type='marcxml')
    assert RecordMetadata.query.count() == 2
    pid, record = resolver.resolve('10')
    assert record['_collections'] == [
        "ALEPH Papers",
        "Articles & Preprints",
        "Experimental Physics (EP)",
        "CERN Divisions",
        "Atlantis Institute of Fictive Science",
        "CERN Experiments",
        "Preprints",
        "ALEPH",
    ]
    assert len(record['_files']) == 2
